# Copyright (c) 2005 Infrae. All rights reserved.
# See also LICENSE.txt
# $Revision: 1.7 $
import os

from Products.SilvaExternalSources.interfaces import IExternalSource
from Products.SilvaExternalSources.ExternalSource import ExternalSource
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.Formulator.Form import ZMIForm, XMLToForm
from Products.Silva.helpers import add_and_edit
from Products.Silva.SilvaPermissions import AccessContentsInformation

from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder


class Facet(ExternalSource):
    """Facets are simple toolsets to allow faceted navigation in Silva. 
    Different facets will be defined in the Silva root that can be called
    from any Silva Document in the hierarchy. Facets will essentially be
    instantiated Code Sources, or at least will implement the same interface.
    """

    meta_type = "Silva Facet"

    security = ClassSecurityInfo()

    _is_cacheable = 1

InitializeClass(Facet)
    
class GetSidebar(Facet):
    """Not really a facet itself, it imports a document containing the
    actual Facets into another document.
    """

    __implements__ = IExternalSource

    meta_type = "Silva Get Sidebar Facet"

    security = ClassSecurityInfo()

    security.declareProtected(AccessContentsInformation, 'to_html')
    def to_html(self, REQUEST, **kw):
        """ render HTML for facet
        """
        doc = self.restrictedTraverse(kw['document_path'].split('/'))
        return '<div class="info">%s</div>' % doc.view()    

    def set_form(self):
        self.parameters = ZMIForm('form', 'Parameters Form')
        f = open(os.path.join(package_home(
            globals()),
            'www',
            'get_sidebar_form.form'))
        XMLToForm(f.read(), self.parameters)
        f.close() 
    
InitializeClass(GetSidebar)

addGetSidebar = PageTemplateFile(
    "www/getSidebarAdd", globals(), __name__='addGetSidebar')
    
def manage_addGetSidebar(context, id='facet_get_sidebar', title='Get Sidebar Facet', script_id=None, REQUEST=None):
    """Add a getSidebar object
    """
    cs = GetSidebar(id, title)
    context._setObject(id, cs)
    cs = context._getOb(id)
    cs.set_form()
    add_and_edit(context, id, REQUEST, screen='editGetSidebar')
    return ''
       
class MultiToc(Facet):
    """The multitool among facets. It allows for multiple tocs, starting
    at arbitrary locations, and can filter by (multiple) meta-types. For 
    instance you could tell it to displat links to all news articles
    and software packages in such and such directories.
    """

    __implements__ = IExternalSource
    
    meta_type = "Silva Multi Toc Facet"

    security = ClassSecurityInfo()

    security.declareProtected(AccessContentsInformation, 'to_html')
    def to_html(self, REQUEST, **kw):
        """ render HTML for facet
        """
        html = []
        add = html.append
        type_list = []
        types = kw.get('filter_meta_types')
        if types:
            type_list = types.split(',')
        depth = kw['depth']
        for path in kw['paths'].split(','):
            model = self.restrictedTraverse(path.split('/'))
            tree = model.get_public_tree(depth=depth)
            add('<h2 class="heading">')
            add(model.get_title())
            add('</h2>')
            add('<p class="toc">')
            for obj in tree:
                indent = obj[0]
                item = obj[1]
                title = obj[1].get_title()
                url = item.absolute_url() + "/"
                if types == None or item.meta_type in type_list:
                    if indent > 0:
                        add('<img width="12" height="14" alt="" width="%s" src="%s/globals/pixel.gif" />' % (str(indent * 24), self.silva_root))
                    add('<a class="toc-link" href="%s">%s</a><br />' % (url, title))
            add('</p>')
        return '\n'.join(html)

    def set_form(self):
        self.parameters = ZMIForm('form', 'Parameters Form')
        f = open(os.path.join(package_home(
            globals()),
            'www',
            'multi_toc_form.form'))
        XMLToForm(f.read(), self.parameters)
        f.close() 
    
InitializeClass(MultiToc)
    
addMultiToc = PageTemplateFile(
    "www/multiTocAdd", globals(), __name__='addMultiToc')

def manage_addMultiToc(context, id='facet_multi_toc', title='Multi TOC Facet', script_id=None, REQUEST=None):
    """Add a MultiToc object
    """
    cs = MultiToc(id, title)
    context._setObject(id, cs)
    cs = context._getOb(id)
    cs.set_form()
    add_and_edit(context, id, REQUEST, screen='editMultiToc')
    return ''

    